@@ -2,7 +2,8 @@ use 5.006;
 use strict;
 use warnings;
 package CPAN::Reporter::Smoker;
-our $VERSION = '0.26'; # VERSION
+
+our $VERSION = '0.28';
 
 use Carp;
 use Config;
@@ -15,6 +16,7 @@ use Fcntl ':flock';
 use File::Basename qw/basename dirname/;
 use File::Spec 3.27;
 use File::Temp 0.20;
+use List::Util 1.03 qw/shuffle/;
 use Probe::Perl 0.01;
 use Term::Title 0.01;
 
@@ -63,6 +65,10 @@ my %spec = (
     default => 0,
     is_valid => sub { /^[01]$/ },
   },
+  'random' => {
+    default => 0,
+    is_valid => sub { /^[01]$/ },
+  },
   force_trust => {
     default => 0,
     is_valid => sub { /^[01]$/ },
@@ -73,12 +79,20 @@ my %spec = (
   },
   filter => {
     default => undef,
-    is_valid => sub { !defined $_ || ref $_ eq 'SUB' }
+    is_valid => sub { !defined $_ || ref $_ eq 'CODE' }
   },
   skip_dev_versions => {
     default => 0,
     is_valid => sub { /^[01]$/ },
   },
+  '_start_from_timestamp' => {
+    default => 0,
+    is_valid => sub { /^(?:[\d.]{8}|0)$/ },
+  },
+  _hook_after_test => {
+    default => undef,
+    is_valid => sub { !defined $_ || ref $_ eq 'CODE' }
+  },
 );
 
 sub start {
@@ -153,7 +167,7 @@ sub start {
       CPAN::Index->reload;
       $CPAN::Frontend->mywarn( "Smoker: scanning and sorting index\n");
 
-      $dists = _parse_module_index( $package, $find_ls, $args{skip_dev_versions} );
+      $dists = _parse_module_index( $package, $find_ls, $args{skip_dev_versions}, $args{_start_from_timestamp} );
 
       $CPAN::Frontend->mywarn( "Smoker: found " . scalar @$dists . " distributions on CPAN\n");
     }
@@ -163,6 +177,11 @@ sub start {
       $dists = [ reverse @$dists ];
     }
 
+    # Maybe shuffle the list
+    if ( $args{'random'} ) {
+      $dists = [ shuffle @$dists ];
+    }
+
     # Check if we need to manually reset test history during each dist loop
     my $reset_string = q{};
     if ( $CPAN::Config->{build_dir_reuse}
@@ -224,7 +243,16 @@ sub start {
           . $reset_string . ($args{'install'} ? 'install' : 'test')
           . "( '$dists->[$d]' )"
         );
-        _prompt_quit( $? & 127 ) if ( $? & 127 );
+        my $interrupted = 0;
+        if ( $? & 127 ) {
+          $interrupted = 1;
+          _prompt_quit( $? & 127 ) ;
+        }
+
+        if ($args{_hook_after_test}) {
+          $args{_hook_after_test}->($dist, $interrupted);
+        }
+        
         # cleanup and record keeping
         unlink $args{status_file} if -f $args{status_file};
         $dists_tested++;
@@ -385,7 +413,7 @@ sub _base_name {
 #--------------------------------------------------------------------------#-
 
 sub _parse_module_index {
-    my ( $packages, $file_ls, $skip_dev_versions ) = @_;
+    my ( $packages, $file_ls, $skip_dev_versions, $start_from_timestamp ) = @_;
 
     # first walk the packages list
     # and build an index
@@ -443,6 +471,8 @@ sub _parse_module_index {
         next unless $stat{name} =~ $re{target_dir};
         next unless $stat{name} =~ $re{archive};
 
+        next if $start_from_timestamp && $stat{datetime} < $start_from_timestamp;
+
         # skip if not AUTHOR/tarball
         # skip perls
         my $base_id = _get_base_id($stat{name});
@@ -450,6 +480,9 @@ sub _parse_module_index {
 
         next if $base_id =~ $re{perls};
 
+        # skip Perl6 distros: AUTHOR/Perl6/...
+        next if $base_id =~ m{\A\w+/Perl6/};
+
         my $base_name = _base_name( $base_id );
 
         # if $base_id matches 02packages, then it is the latest version
@@ -487,10 +520,14 @@ sub _parse_module_index {
         }
     }
 
-    # pick up anything from packages that wasn't found find-ls
-    for my $name ( keys %latest ) {
-        my $base_id = $latest{$name}{base_id};
-        $mirror{$base_id} = $latest{$name}{datetime} unless $mirror{$base_id};
+    if ( !$start_from_timestamp ) {
+        # pick up anything from packages that wasn't found in find-ls
+        # usually because find-ls is updated more rarely than packages
+        # as it is missing from find-ls, timestamp would be set to 0
+        for my $name ( keys %latest ) {
+            my $base_id = $latest{$name}{base_id};
+            $mirror{$base_id} = $latest{$name}{datetime} unless $mirror{$base_id};
+        }
     }
 
     # for dev versions, it must be newer than the latest version of
@@ -523,7 +560,7 @@ CPAN::Reporter::Smoker - Turnkey CPAN Testers smoking
 
 =head1 VERSION
 
-version 0.26
+version 0.28
 
 =head1 SYNOPSIS
 
@@ -579,6 +616,10 @@ Current limitations:
 Does not attempt to retest distributions that had reports discarded because
 of prerequisites that could not be satisfied
 
+=item *
+
+Does not test trial version if there is no non-trial version
+
 =back
 
 =head2 WARNING -- smoke testing is risky
@@ -657,6 +698,11 @@ distribution if --list is provided). Valid values are 0 or 1. Defaults to 0
 
 =item *
 
+C<<< random >>> -- toggle whether to randomize distribution test order. When set to 1,
+the list of releases is shuffled. Valid values are 0 or 1. Defaults to 0
+
+=item *
+
 C<<< force_trust >>> -- toggle whether to override CPAN's
 C<<< trust_test_report_history >>> option. When set to 1, C<<< trust_test_report_history >>>
 is set to 1.  When set to 0, C<<< trust_test_report_history >>> is left alone and
@@ -1002,6 +1048,8 @@ David Golden <dagolden@cpan.org>
 
 =head1 CONTRIBUTORS
 
+=for stopwords Alexandr Ciornii Christian Walde gregor herrmann
+
 =over 4
 
 =item *
@@ -1012,11 +1060,15 @@ Alexandr Ciornii <alexchorny@gmail.com>
 
 Christian Walde <walde.christian@googlemail.com>
 
+=item *
+
+gregor herrmann <gregoa@cpan.org>
+
 =back
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by David Golden.
+This software is Copyright (c) 2016 by David Golden.
 
 This is free software, licensed under:
 
